/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2010  Rico Rommel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SQLRESOURCE_H
#define SQLRESOURCE_H

#include <akonadi/resourcebase.h>
#include <QSqlDatabase>
#include <QSqlQuery>
#include <QStringList>
#include <QSqlRecord>
#include <QSqlField>
#include <QDBusConnection>
#include <kabc/phonenumber.h>
#include <kabc/address.h>
#include "types.h"
#include <kabc/addressee.h>
#include <kwallet.h>
#include "logindialog.h"


class SQLResource : public Akonadi::ResourceBase,
            public Akonadi::AgentBase::Observer
{
    Q_OBJECT

public:
    SQLResource( const QString &id );
    ~SQLResource();

public Q_SLOTS:
    virtual void configure( WId windowId );

protected Q_SLOTS:
    void retrieveCollections();
    void retrieveItems( const Akonadi::Collection &col );
    bool retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts );

protected:
    QSqlDatabase db;
    QMap<KABC::PhoneNumber::TypeFlag, QString> phonetypes;
    QMap<KABC::Address::TypeFlag, QString> addresstypes;
    QString ignore;
    QList<AddresseeTypes::Types> MultipleFields;
    struct dbsettings DBSettings;
    QMap<AddresseeTypes::Types, struct typeattributes> TypeAttributesMap;
    QMap<KABC::Address::TypeFlag, QList <AddresseeTypes::Types> > AddressTypeMap;
    QMap<KABC::PhoneNumber::TypeFlag, AddresseeTypes::Types> PhoneTypeMap;
    bool Online;
    QString KWalletFolder ;
    KWallet::Wallet *wallet;
    int KWalletAccessType;
    LoginDialog *LoginDlg;
    
    virtual void aboutToQuit();
    virtual void itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection );
    virtual void itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &parts );
    virtual void itemRemoved( const Akonadi::Item &item );
    void saveSettings();
    void readSettings();
    void createTable( QString table );
    bool tableExists( QString table );
    void initDB();
    KABC::Address createAddress(KABC::Address::TypeFlag type, QList<AddresseeTypes::Types> list, QSqlRecord record);
    void createTypeAttributesMap();
    void createAddressTypeMap();
    void createPhoneTypeMap();
    bool createNewField(AddresseeTypes::Types type, QString suffix ="");
    bool createNewField( QString fieldname, AddresseeTypes::Types type, QString suffix ="");
    void writeToDatabase(KABC::Addressee addressee, QString id);
    KABC::Addressee readFromDatabase(QString id);
    bool writeValueToDataBase(QString col, QString val, QString id);
    bool writeBlobToDataBase(QString col, QByteArray val, QString id);
    void addTypeAttributesToMap(AddresseeTypes::Types type, QString text, QString dbtype);
    QString makeStringDataBaseCompliant(QString name);
    QString findNextFieldSuffix(QString name);
    QStringList getOrderedFieldList(AddresseeTypes::Types type);
    void getDatabaseLoginFromKWallet();
    void getDatabaseLoginFromDialog();
    void saveDatabaseLoginToKWallet();
    void openKWallet();
    
protected slots:
  void accessKWallet(bool ok);
  void readLoginDataFromDialog();
  void gotLoginData();


};


#endif
